import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

public class Main {
    /**
     * Функция для проверки требуемых аргументов программы
     *
     * @param args аргументы программы
     */
    private static boolean checkArgs(String[] args) {
        if (args.length == 0) {
            System.out.println("Передайте путь к файлу");
            return false;
        }

        if (args.length == 1 || args[args.length - 1].length() > 1) {
            System.out.println("Передайте букву для поиска");
            return false;
        }

        return true;
    }

    /**
     * Функция удаляем все знаки препинания в конце слова
     *
     * @param word Слово
     * @return слово без знаков препинания
     */
    private static String removeSymbols(String word) {
        word = word.replace(",", "");
        word = word.replace("!", "");
        word = word.replace(":", "");
        word = word.replace("?", "");
        word = word.replace(".", "");
        return word;
    }

    public static void main(String[] args) throws IOException {
        if (!checkArgs(args)) {
            return;
        }

        String fileName = args[0];
        String symbol = args[1];

        System.out.println("Список слов удовлетворяющие условию задачи");
        List<String> words = Files.lines(Paths.get(fileName)) // будем читать с файла
                .map(line -> line.split("\\s+")) // делаем разбиение на слова. Делителем будем считать пробельный символ
                .flatMap(Arrays::stream) // Преобразуем полученный массив слов в поток и объединяем эти потоки в один
                .map(Main::removeSymbols) // удаляем знаки препинания
                .filter(word -> word.length() >= 5) // фильтруем так чтобы остались только слова, длина которых не меньше 5
                .filter(word -> word.contains(symbol)) // из оставшихся слов выбираем только те, которые содержат заданную букву
                .sorted() // сортируем слова в алфавитном порядке
                .distinct() // убираем дублирубщие слова
                .limit(10) // берем первые 10 слов
                .map(String::toLowerCase) // приводим к нижниму регистру
                .collect(Collectors.toList()); // перемещаем полученные слова из потока в список

        if (words.size() == 0) {
            System.out.println("В файле нет требуемых слов");
        } else {
            words.forEach(System.out::println); // выводим найденные слова
        }
    }
}
